@@ -16,35 +16,35 @@ This product provides support for Script objects containing restricted
 Python code.
 """
 
-from logging import getLogger
-from six.moves.urllib.parse import quote
 import marshal
 import os
 import re
-import six
 import sys
 import types
+from logging import getLogger
+
+import six
+from six.moves.urllib.parse import quote
 
 from AccessControl.class_init import InitializeClass
 from AccessControl.requestmethod import requestmethod
 from AccessControl.SecurityInfo import ClassSecurityInfo
 from AccessControl.SecurityManagement import getSecurityManager
-from AccessControl.ZopeGuards import get_safe_globals, guarded_getattr
+from AccessControl.ZopeGuards import get_safe_globals
+from AccessControl.ZopeGuards import guarded_getattr
 from Acquisition import aq_parent
 from App.Common import package_home
 from App.Dialogs import MessageDialog
 from App.special_dtml import DTMLFile
-from DateTime.DateTime import DateTime
 from OFS.Cache import Cacheable
-from OFS.History import Historical
-from OFS.History import html_diff
 from OFS.SimpleItem import SimpleItem
 from RestrictedPython import compile_restricted_function
 from Shared.DC.Scripts.Script import BindingsUI
-from Shared.DC.Scripts.Script import defaultBindings
 from Shared.DC.Scripts.Script import Script
+from Shared.DC.Scripts.Script import defaultBindings
 from zExceptions import Forbidden
 
+
 try:
     from zExceptions import ResourceLockedError
 except ImportError:
@@ -53,49 +53,54 @@ except ImportError:
 LOG = getLogger('PythonScripts')
 
 # Track the Python bytecode version
-import imp  # NOQA
-Python_magic = imp.get_magic()
-del imp
+try:
+    import importlib.util
+    Python_magic = importlib.util.MAGIC_NUMBER
+    del importlib.util
+except ImportError:  # Python 2
+    import imp
+    Python_magic = imp.get_magic()
+    del imp
 
 # This should only be incremented to force recompilation.
-Script_magic = 3
+Script_magic = 4
 _log_complaint = (
     'Some of your Scripts have stale code cached.  Since Zope cannot'
     ' use this code, startup will be slightly slower until these Scripts'
     ' are edited. You can automatically recompile all Scripts that have'
     ' this problem by visiting /manage_addProduct/PythonScripts/recompile'
     ' of your server in a browser.')
-
 manage_addPythonScriptForm = DTMLFile('www/pyScriptAdd', globals())
-_default_file = os.path.join(package_home(globals()),
-                             'www', 'default_py')
-
+_default_file = os.path.join(package_home(globals()), 'www', 'default_content')
 _marker = []  # Create a new marker object
 
 
-def manage_addPythonScript(self, id, REQUEST=None, submit=None):
+def manage_addPythonScript(self, id, title='', REQUEST=None, submit=None):
     """Add a Python script to a folder.
     """
     id = str(id)
     id = self._setObject(id, PythonScript(id))
+    pyscript = self._getOb(id)
+    if title:
+        pyscript.ZPythonScript_setTitle(title)
     if REQUEST is not None:
         file = REQUEST.form.get('file', '')
         if not isinstance(file, str):
             file = file.read()
         if not file:
             file = open(_default_file).read()
-        self._getOb(id).write(file)
+        pyscript.write(file)
         try:
             u = self.DestinationURL()
         except Exception:
             u = REQUEST['URL1']
-        if submit == " Add and Edit ":
-            u = "%s/%s" % (u, quote(id))
+        if submit == 'Add and Edit':
+            u = '%s/%s' % (u, quote(id))
         REQUEST.RESPONSE.redirect(u + '/manage_main')
     return ''
 
 
-class PythonScript(Script, Historical, Cacheable):
+class PythonScript(Script, Cacheable):
     """Web-callable scripts written in a safe subset of Python.
 
     The function may include standard python code, so long as it does
@@ -103,6 +108,7 @@ class PythonScript(Script, Historical, Cacheable):
     """
 
     meta_type = 'Script (Python)'
+    zmi_icon = 'fa fa-terminal'
     _proxy_roles = ()
 
     _params = _body = ''
@@ -114,7 +120,7 @@ class PythonScript(Script, Historical, Cacheable):
     ) + BindingsUI.manage_options + (
         {'label': 'Test', 'action': 'ZScriptHTML_tryForm'},
         {'label': 'Proxy', 'action': 'manage_proxyForm'},
-    ) + Historical.manage_options + SimpleItem.manage_options + \
+    ) + SimpleItem.manage_options + \
         Cacheable.manage_options
 
     def __init__(self, id):
@@ -125,9 +131,9 @@ class PythonScript(Script, Historical, Cacheable):
     security = ClassSecurityInfo()
 
     security.declareObjectProtected('View')
-    security.declareProtected('View', '__call__')
+    security.declareProtected('View', '__call__')  # NOQA: flake8: D001
 
-    security.declareProtected(
+    security.declareProtected(  # NOQA: flake8: D001
         'View management screens',
         'ZPythonScriptHTML_editForm', 'manage_main', 'read',
         'ZScriptHTML_tryForm', 'PrincipiaSearchSource',
@@ -137,17 +143,17 @@ class PythonScript(Script, Historical, Cacheable):
     manage = manage_main = ZPythonScriptHTML_editForm
     ZPythonScriptHTML_editForm._setName('ZPythonScriptHTML_editForm')
 
-    security.declareProtected(
+    security.declareProtected(  # NOQA: flake8: D001
         'Change Python Scripts',
         'ZPythonScriptHTML_editAction',
         'ZPythonScript_setTitle', 'ZPythonScript_edit',
-        'ZPythonScriptHTML_upload', 'ZPythonScriptHTML_changePrefs')
+        'ZPythonScriptHTML_upload')
 
     def ZPythonScriptHTML_editAction(self, REQUEST, title, params, body):
         """Change the script's main parameters."""
         self.ZPythonScript_setTitle(title)
         self.ZPythonScript_edit(params, body)
-        message = "Saved changes."
+        message = 'Saved changes.'
         return self.ZPythonScriptHTML_editForm(self, REQUEST,
                                                manage_tabs_message=message)
 
@@ -160,7 +166,7 @@ class PythonScript(Script, Historical, Cacheable):
     def ZPythonScript_edit(self, params, body):
         self._validateProxy()
         if self.wl_isLocked():
-            raise ResourceLockedError("The script is locked via WebDAV.")
+            raise ResourceLockedError('The script is locked via WebDAV.')
         if not isinstance(body, str):
             body = body.read()
 
@@ -171,11 +177,14 @@ class PythonScript(Script, Historical, Cacheable):
     def ZPythonScriptHTML_upload(self, REQUEST, file=''):
         """Replace the body of the script with the text in file."""
         if self.wl_isLocked():
-            raise ResourceLockedError("The script is locked via WebDAV.")
+            raise ResourceLockedError('The script is locked via WebDAV.')
 
         if not isinstance(file, str):
             if not file:
-                raise ValueError('File not specified')
+                return self.ZPythonScriptHTML_editForm(
+                    self, REQUEST,
+                    manage_tabs_message='No file specified',
+                    manage_tabs_type='warning')
             file = file.read()
 
         self.write(file)
@@ -183,49 +192,20 @@ class PythonScript(Script, Historical, Cacheable):
         return self.ZPythonScriptHTML_editForm(self, REQUEST,
                                                manage_tabs_message=message)
 
-    def ZPythonScriptHTML_changePrefs(self, REQUEST, height=None, width=None,
-                                      dtpref_cols="100%", dtpref_rows="20"):
-        """Change editing preferences."""
-        dr = {"Taller": 5, "Shorter": -5}.get(height, 0)
-        dc = {"Wider": 5, "Narrower": -5}.get(width, 0)
-        if isinstance(height, int):
-            dtpref_rows = height
-        if (isinstance(width, int) or
-                isinstance(width, str) and width.endswith('%')):
-            dtpref_cols = width
-        rows = str(max(1, int(dtpref_rows) + dr))
-        cols = str(dtpref_cols)
-        if cols.endswith('%'):
-            cols = str(min(100, max(25, int(cols[:-1]) + dc))) + '%'
-        else:
-            cols = str(max(35, int(cols) + dc))
-        e = (DateTime("GMT") + 365).rfc822()
-        setCookie = REQUEST["RESPONSE"].setCookie
-        setCookie("dtpref_rows", rows, path='/', expires=e)
-        setCookie("dtpref_cols", cols, path='/', expires=e)
-        REQUEST.other.update({"dtpref_cols": cols, "dtpref_rows": rows})
-        return self.manage_main(self, REQUEST)
-
     def ZScriptHTML_tryParams(self):
         """Parameters to test the script with."""
         param_names = []
         for name in self._params.split(','):
 
             name = name.strip()
-            if name and name[0] != '*' and re.match('\w', name):
+            if name and name[0] != '*' and re.match(r'\w', name):
                 param_names.append(name.split('=', 1)[0].strip())
         return param_names
 
-    def manage_historyCompare(self, rev1, rev2, REQUEST,
-                              historyComparisonResults=''):
-        return PythonScript.inheritedAttribute('manage_historyCompare')(
-            self, rev1, rev2, REQUEST,
-            historyComparisonResults=html_diff(rev1.read(), rev2.read()))
-
     def __setstate__(self, state):
         Script.__setstate__(self, state)
-        if (getattr(self, 'Python_magic', None) != Python_magic or
-                getattr(self, 'Script_magic', None) != Script_magic):
+        if getattr(self, 'Python_magic', None) != Python_magic or \
+           getattr(self, 'Script_magic', None) != Script_magic:
             global _log_complaint
             if _log_complaint:
                 LOG.info(_log_complaint)
@@ -389,7 +369,7 @@ class PythonScript(Script, Historical, Cacheable):
             'because you do not have proxy roles.\n<!--%s, %s-->' % (
                 self.id, user, roles))
 
-    security.declareProtected(
+    security.declareProtected(  # NOQA: flake8: D001
         'Change proxy roles',
         'manage_proxyForm', 'manage_proxy')
 
@@ -397,7 +377,7 @@ class PythonScript(Script, Historical, Cacheable):
 
     @requestmethod('POST')
     def manage_proxy(self, roles=(), REQUEST=None):
-        "Change Proxy Roles"
+        """Change Proxy Roles"""
         self._validateProxy(roles)
         self._validateProxy()
         self.ZCacheable_invalidate()
@@ -408,11 +388,9 @@ class PythonScript(Script, Historical, Cacheable):
                 message='Your changes have been saved',
                 action='manage_main')
 
-    security.declareProtected(
+    security.declareProtected(  # NOQA: flake8: D001
         'Change Python Scripts',
-        'PUT', 'manage_FTPput', 'write',
-        'manage_historyCopy',
-        'manage_beforeHistoryCopy', 'manage_afterHistoryCopy')
+        'PUT', 'manage_FTPput', 'write')
 
     def PUT(self, REQUEST, RESPONSE):
         """ Handle HTTP PUT requests """
@@ -478,12 +456,12 @@ class PythonScript(Script, Historical, Cacheable):
                 self.ZBindings_edit(bindmap)
             else:
                 self._makeFunction()
-        except:
+        except Exception:
             LOG.error('write failed', exc_info=sys.exc_info())
             raise
 
     def manage_FTPget(self):
-        "Get source for FTP download"
+        """Get source for FTP download"""
         self.REQUEST.RESPONSE.setHeader('Content-Type', 'text/plain')
         return self.read()
 
@@ -539,8 +517,8 @@ class PythonScript(Script, Historical, Cacheable):
     getSize = get_size
 
     def PrincipiaSearchSource(self):
-        "Support for searching - the document's contents are searched."
-        return "%s\n%s" % (self._params, self._body)
+        """Support for searching - the document's contents are searched."""
+        return '%s\n%s' % (self._params, self._body)
 
     def document_src(self, REQUEST=None, RESPONSE=None):
         """Return unprocessed document source."""
@@ -562,7 +540,7 @@ class PythonScriptTracebackSupplement:
 
 
 _first_indent = re.compile('(?m)^ *(?! |$)')
-_nonempty_line = re.compile('(?m)^(.*\S.*)$')
+_nonempty_line = re.compile(r'(?m)^(.*\S.*)$')
 
 _nice_bind_names = {'context': 'name_context', 'container': 'name_container',
                     'script': 'name_m_self', 'namespace': 'name_ns',
